-- $Id: rs.rex 112 2024-03-18 12:33:40Z jeffr $

parse arg data year month size_factor title

if data = '' | year = '' | month = '' | DATATYPE(year) \= NUM | DATATYPE(month) \= CHAR | length(year) > 2 then do
	say 'Usage:   rs.rex {filename} {yy} {mmm or -} {width (optional)} {title (optional)}'
	say 'Example: rs.rex data.csv 23 dec 1 December 2023 Runs'
	exit 1
end

if state(data) = 1 then do
	say 'rs.rex error: data file not found.'
	exit 1
end

obs = 0
runs = 0
tt = 0
td = 0
max_d = 0
min_d = 99
eg = 0
notes_c = 0
hdr_w = 75

-- Main loop
do while lines(data) > 0
	wo = linein(data)
	parse value wo with date ',' type ',' loc ',' time ',' equip ',' dist ',' elev ',' note
	parse value date with dd '-' mmm '-' yy

	-- Full year or month?
	if month = '-' then do
		-- Year
		if yy = year then
			calcs = calc()
	end
	else do
		-- Month
		if yy = year & upper(mmm) = upper(month) then
			calcs = calc()
	end
-- End main loop
end

-- Exit if no data for period
if runs = 0 then do
	say 'No data for specified period.'
	exit 1
end

-- Show report
if title = '' then
	title = 'Running Statistics'

say
say center(title, hdr_w, '-')
say
say 'Total entries:  'obs
say 'Total runs:     'runs
say 'Total time:     'format(tt/60,,2) 'hours'
say 'Avg. time:      'format(tt/runs,,2) 'minutes'
say 'Total distance: 'td 'miles'
say 'Avg. distance:  'format(td/runs,,2) 'miles'
say 'Avg. pace:      'format(tt/td,,2) 'minutes per mile'
say 'Min. distance:  'min_d 'miles'
say 'Max. distance:  'max_d 'miles'
say 'Elevation gain: 'eg 'feet'

say

-- Determine ranges
std_max = 15
rng_val = std_max/5
rng_u = 0
do i = 1 to 5
	rng_u = rng_u + rng_val
	rng.i = rng_u
end

rng_1_c = 0
rng_2_c = 0
rng_3_c = 0
rng_4_c = 0
rng_5_c = 0

-- Calculate values for the ranges
do i = 1 to runs
	if dist_vals.i < rng.1 then
		do
			rng_1_c = rng_1_c + 1 
		end
	else if dist_vals.i < rng.2 then
		do
			rng_2_c = rng_2_c + 1
		end
	else if dist_vals.i < rng.3 then
		do
			rng_3_c = rng_3_c + 1
		end
	else if dist_vals.i < rng.4 then
		do
			rng_4_c = rng_4_c + 1
		end
	else
		rng_5_c = rng_5_c + 1
end

if size_factor = '' | DATATYPE(size_factor) \= NUM then
	size_factor = 1.0

rng_1_c_b = trunc(rng_1_c * size_factor)
rng_2_c_b = trunc(rng_2_c * size_factor)
rng_3_c_b = trunc(rng_3_c * size_factor)
rng_4_c_b = trunc(rng_4_c * size_factor)
rng_5_c_b = trunc(rng_5_c * size_factor)

say center('Distance Profile', hdr_w, '-')
say
say left('<'rng.1, 8,' ') right(rng_1_c, rng_1_c_b, '*')
say left(rng.1' to <'rng.2,8,' ') right(rng_2_c, rng_2_c_b, '*')
say left(rng.2' to <'rng.3,8,' ') right(rng_3_c, rng_3_c_b, '*')
say left(rng.3' to <'rng.4,8,' ') right(rng_4_c, rng_4_c_b, '*')
say left(rng.4'+', 8, ' ') right(rng_5_c, rng_5_c_b, '*')
say

-- Notes
say center('Notes', hdr_w, '-')
say
if notes_c > 0 then do
	do i = 1 to notes_c
		parse value notes.i with date ',' note
		say date': 'note
	end
end
else
	say 'No notes.'
	
say
exit 0

-- Calculate running stats and get notes, distances
calc:
obs = obs + 1
if upper(type) = 'RUN' then do
	runs = runs + 1
	tt = tt + time
	td = td + dist
	eg = eg + elev

	-- Minimum distance
	if dist <= min_d then
		min_d = dist
	-- Maximum distance
	if dist >= max_d then
		max_d = dist
	
	if note \= '' then do
		notes_c = notes_c + 1
		notes.notes_c = date','note
	end

	dist_vals.runs = dist
end
return obs
